'
' TIMER3 MACROS
'
' For 16-bit core (18F) devices using version 3.1 onwards of the PROTON+ Compiler
'
' Interrupt bit mask to be 'anded' with the other configuration masks and
' passed as the PARAM_CONFIG parameter to the 'open' macros.
'
	Symbol T3_INT_OFF   = %01111111   			' Interrupts disabled
	Symbol T3_INT_ON    = %11111111   			' Interrupts enabled
'
' TIMER3 configuration masks -- to be 'anded' together and passed to the 'open' macro
'
' Timer Width
'
	Symbol T3_8BIT_RW       = %11111110 		' 8-bit mode
	Symbol T3_16BIT_RW      = %11111111  		' 16-bit mode
'
' Clock Source
'
	Symbol T3_SOURCE_INT    = %11111101  		' Internal clock source
	Symbol T3_SOURCE_EXT    = %11111111  		' External clock source
'
' Prescaler
'
	Symbol T3_PS_1_1        = %11001111  		' 1:1 prescale value
	Symbol T3_PS_1_2        = %11011111  		' 1:2 prescale value
	Symbol T3_PS_1_4        = %11101111  		' 1:4 prescale value
	Symbol T3_PS_1_8        = %11111111  		' 1:8 prescale value
'
' Oscillator Use
'
	Symbol T3_OSC1EN_OFF    = %11111101  		' Timer 3 oscillator enable off
	Symbol T3_OSC1EN_ON     = %11111111  		' Timer 3 oscillator enable on
'
' Synchronize Clock Input
'
	Symbol T3_SYNC_EXT_ON   = %11111011  		' Synchronize external clock input
	Symbol T3_SYNC_EXT_OFF  = %11111111  		' Do not synchronize external clock input
'
' T3 and T4 are sources for CCP1 thru CCP5
'
	Symbol T34_SOURCE_CCP   	 = %11111111 	
'
' T1 and T2 are sources for CCP1 and CCP2 and 
' T3 and T4 are sources for CCP3 thru CCP5
'
	Symbol T12_CCP12_T34_CCP345	 = %11110111 
'
' T1 and T2 are sources for CCP1 and 
' T3 and T4 are sources for CCP2 thru CCP5
'
	Symbol T12_CCP1_T34_CCP2345	 = %10111111 
'
' T1 and T2 are sources for CCP1 thru CCP5
'
	Symbol T12_SOURCE_CCP   	 = %10110111 
'
' T3 is source for CCP
'
	Symbol T3_SOURCE_CCP         = %11111111 
'
' T1 is source for CCP1 and T3 is source for CCP2
'
	Symbol T1_CCP1_T3_CCP2       = %10111111 
'
' T1 is source for CCP
'
	Symbol T1_SOURCE_CCP         = %10110111 

'--------------------------------------------------------------------------------- 
' Macro Name	: OPEN_TIMER3                                     
' Return Value	: None                                           
' Parameters	: config: bit definitions to configure Timer3    
' Description	: This routine first resets the Timer3 regs to the POR state and then configures the interrupt, clock source.                       
' Notes			: The bit definitions for config can be found in the timers.h file.                          
'                                                                   
'                 The MPLAB CXX Reference Guide says that Open_Timer1(config) can select Timer 1 as the source for the CCPs.  
'				  However, this function does not write the necessary values to bits 6 and 3 of T3CON.
'				  Instead, the function incorrectly affects bit 3 of T1CON.            
'                                                                   
'                 Because the power-on default is timer1 source for both CCPs, we do not need to even have these values.
'				  Should a user want to change this value, the user would need select the appropriate value when calling OpenTimer3.     
'                                                                   
OPEN_TIMER3 MACRO PARAM_CONFIG
	#if(PRM_COUNT != 1)
    	#error "OPEN_TIMER3 Macro requires a parameter"
    	EXITM
    #endif
    #if((PRM_1 != NUM8) && (PRM_1 != NUM16) && (PRM_1 != NUM32))
    	#error "OPEN_TIMER3 Macro parameter must be a constant value"
    	EXITM
    #endif
    #if((0X7E & PARAM_CONFIG) == 0)
    	Clrf T3CON
    #else
    	Movlw (0X7E & PARAM_CONFIG)	
  		Movwf T3CON						' Set everything except 8/16 mode, and don't start timer yet
  	#endif     
  	TMR3H = 0          					' Clear timer3-related registers
  	TMR3L = 0
  	Bcf PIR2,TMR3IF
  	#if(PARAM_CONFIG & 0X80)   			' Interrupt on/off
    	Bsf PIE2,TMR3IE
  	#else
    	Bcf PIE2,TMR3IE
	#endif  
  	#if(PARAM_CONFIG & 2)
    	Bsf T1CON,T1OSCEN  				' Enable Timer 1 Oscillator enable (200KHz max freq)
	#endif   
  	#if(PARAM_CONFIG & ~T3_8BIT_RW) 	' Select between 8-bit and 16-bit modes
    	Bsf T3CON,RD16
	#else
    	Bcf T3CON,RD16
	#endif
  	Bsf T3CON,TMR3ON   					' Turn on Timer3
  	ENDM
'---------------------------------------------------------------------------------  
' Macro Name 	: CLOSE_TIMER3                                    
' Return Value	: None                                           
' Parameters	: None                                           
' Description	: Disable the Timer3 interrupt and stop Timer0    
'
CLOSE_TIMER3 MACRO
    Bcf T3CON,TMR3ON				' Disable Timer3
  	Bcf PIE2,TMR3IE					' Disable Timer3 overflow interrupts
    ENDM      
'---------------------------------------------------------------------------------     
' Macro Name  	: READ_TIMER3                                     
' Return Value	: Timer3 8-bit or 16-bit value                       
' Parameters	: Size of Timer3 8 or 16                                           
' Description	: Read an 8 or 16-bit value from Timer3.                                        
'
READ_TIMER3 MACRO PARAM_SIZE
	#if(PRM_COUNT != 1)
    	#error "READ_TIMER3 Macro requires a parameter of 8 or 16"
    	EXITM
    #endif
    #if((PRM_1 != NUM8) && (PRM_1 != NUM16) && (PRM_1 != NUM32))
    	#error "READ_TIMER3 Macro parameter must be a constant value of 8 or 16"
    	EXITM
    #endif
    #if(PARAM_SIZE == 8)
    	#if(READ_TIMER3_RETURN == 1)
        	#if((RETURN_TYPE == BYTE) || (RETURN_TYPE == WORD) || (RETURN_TYPE == DWORD))
        		#if(RETURN_VAR == WREG)
                	Movfw TMR3L
                #else
                	Byte_Byte TMR3L,RETURN_VAR  	' Copy Timer3 low byte
  				#endif
            #endif
        #endif
    #else
  		#if(READ_TIMER3_RETURN == 1)
        	#if(RETURN_TYPE == BYTE)
        		#if(RETURN_VAR == WREG)
                	Movfw TMR3L
                #else
                	Byte_Byte TMR3L,RETURN_VAR  	' Copy Timer3 low byte
  				#endif
  			#endif
        	#if(RETURN_TYPE == WORD)
        		Byte_Byte TMR3L,RETURN_VAR  		' Copy Timer3 low byte
  				Byte_Byte TMR3H,RETURN_VAR + 1  	' Copy Timer3 high byte
        	#endif 
		#endif
    #endif
  	ENDM 
'---------------------------------------------------------------------------------
' Macro Name  	: WRITE_TIMER3                                    
' Return Value	: None                                           
' Parameters	: Size of Timer3 8 or 16
' 				: Value to write to Timer3                  
' Description	: Write a 16-bit value to Timer3                                           
'
WRITE_TIMER3 MACRO PARAM_SIZE, PARAM_TIMER

	#if(PRM_COUNT != 2)
    	#error "WRITE_TIMER3 Macro requires 2 parameters of 8 or 16, and the value placed into TIMER3"
    	EXITM
    #endif
    #if((PRM_1 != NUM8) && (PRM_1 != NUM16) && (PRM_1 != NUM32))
    	#error "WRITE_TIMER3 Macro parameter 1 must be a constant value of 8 or 16"
    	EXITM
    #endif
    
    #if(PRM_2 == NUM8)
    	#if(PARAM_SIZE == 8)
        	#if(PARAM_TIMER == 0)
            	Clrf TMR3L
            #else
            	Movlw (PARAM_TIMER)
            	Movwf TMR3L
            #endif
        #else
            Clrf TMR3H
            #if((PARAM_TIMER & 255) == 0)
            	Clrf TMR3L
            #else
            	Movlw (PARAM_TIMER & 255)
            	Movwf TMR3L
            #endif
        #endif
    #endif
    #if((PRM_2 == NUM16) || (PRM_2 == NUM32))
    	#if(PARAM_SIZE == 8)
            #if((PARAM_TIMER & 255) == 0)
            	Clrf TMR3L
            #else
            	Movlw (PARAM_TIMER & 255)
            	Movwf TMR3L
            #endif
        #else
            #if(((PARAM_TIMER >> 8) & 255) == 0)
            	Clrf TMR3H
            #else
            	Movlw (PARAM_TIMER >> 8)
            	Movwf TMR3H
            #endif
            #if((PARAM_TIMER & 255) == 0)
            	Clrf TMR3L
            #else
            	Movlw (PARAM_TIMER & 255)
            	Movwf TMR3L
            #endif
        #endif
    #endif
    #if(PRM_2 == BYTE)
    	#if(PARAM_SIZE == 8)
        	#if(PARAM_TIMER == WREG)
            	Movwf TMR3L
            #else
        		Byte_Byte PARAM_TIMER,TMR3L
            #endif
        #else
        	Clrf TMR3H
            Byte_Byte PARAM_TIMER,TMR3L
        #endif
    #endif
    #if((PRM_2 == WORD) || (PRM_2 == DWORD))
    	#if(PARAM_SIZE == 8)
        	#if(PARAM_TIMER == WREG)
            	Movwf TMR3L
            #else
        		Byte_Byte PARAM_TIMER,TMR3L
            #endif
        #else
        	Byte_Byte PARAM_TIMER + 1,TMR3H
            Byte_Byte PARAM_TIMER,TMR3L
        #endif
    #endif
    ENDM 